@@ -11,10 +11,20 @@
 
 import os
 import re
+from collections import namedtuple
 from os import path
 from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
-from datetime import datetime
-from collections import namedtuple
+
+from docutils import nodes
+from docutils.utils import smartquotes
+
+from sphinx import addnodes
+from sphinx.builders.html import BuildInfo, StandaloneHTMLBuilder
+from sphinx.util import logging
+from sphinx.util import status_iterator
+from sphinx.util.fileutil import copy_asset_file
+from sphinx.util.i18n import format_date
+from sphinx.util.osutil import ensuredir, copyfile
 
 try:
     from PIL import Image
@@ -24,16 +34,6 @@ except ImportError:
     except ImportError:
         Image = None
 
-from docutils import nodes
-from docutils.utils import smartquotes
-
-from sphinx import addnodes
-from sphinx.builders.html import StandaloneHTMLBuilder
-from sphinx.util import logging
-from sphinx.util import status_iterator
-from sphinx.util.osutil import ensuredir, copyfile
-from sphinx.util.fileutil import copy_asset_file
-
 if False:
     # For type annotation
     from typing import Any, Dict, List, Tuple  # NOQA
@@ -159,6 +159,10 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.id_cache = {}  # type: Dict[unicode, unicode]
         self.use_index = self.get_builder_config('use_index', 'epub')
 
+    def create_build_info(self):
+        # type: () -> BuildInfo
+        return BuildInfo(self.config, self.tags, ['html', 'epub'])
+
     def get_theme_config(self):
         # type: () -> Tuple[unicode, Dict]
         return self.config.epub_theme, self.config.epub_theme_options
@@ -486,7 +490,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata['copyright'] = self.esc(self.config.epub_copyright)
         metadata['scheme'] = self.esc(self.config.epub_scheme)
         metadata['id'] = self.esc(self.config.epub_identifier)
-        metadata['date'] = self.esc(datetime.utcnow().strftime("%Y-%m-%d"))
+        metadata['date'] = self.esc(format_date("%Y-%m-%d"))
         metadata['manifest_items'] = []
         metadata['spines'] = []
         metadata['guides'] = []
@@ -513,6 +517,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if not self.use_index:
             self.ignored_files.append('genindex' + self.out_suffix)
         for root, dirs, files in os.walk(outdir):
+            dirs.sort()
             for fn in sorted(files):
                 filename = path.join(root, fn)[olen:]
                 if filename in self.ignored_files:
@@ -667,7 +672,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         metadata = {}  # type: Dict[unicode, Any]
         metadata['uid'] = self.config.epub_uid
-        metadata['title'] = self.config.epub_title
+        metadata['title'] = self.esc(self.config.epub_title)
         metadata['level'] = level
         metadata['navpoints'] = navpoints
         return metadata
