@@ -7,7 +7,7 @@
     Classes for docstring parsing and formatting.
 
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,57 +15,65 @@ import collections
 import inspect
 import re
 
-from six import string_types
+from six import string_types, u
 from six.moves import range
 
 from sphinx.ext.napoleon.iterators import modify_iter
 from sphinx.util.pycompat import UnicodeMixin
 
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Tuple, Union  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config as SphinxConfig  # NOQA
+
 
 _directive_regex = re.compile(r'\.\. \S+::')
 _google_section_regex = re.compile(r'^(\s|\w)+:\s*$')
-_google_typed_arg_regex = re.compile(r'\s*(.+?)\s*\(\s*(.+?)\s*\)')
+_google_typed_arg_regex = re.compile(r'\s*(.+?)\s*\(\s*(.*[^\s]+)\s*\)')
 _numpy_section_regex = re.compile(r'^[=\-`:\'"~^_*+#<>]{2,}\s*$')
-_xref_regex = re.compile(r'(:\w+:\S+:`.+?`|:\S+:`.+?`|`.+?`)')
+_single_colon_regex = re.compile(r'(?<!:):(?!:)')
+_xref_regex = re.compile(r'(:(?:[a-zA-Z0-9]+[\-_+:.])*[a-zA-Z0-9]+:`.+?`)')
+_bullet_list_regex = re.compile(r'^(\*|\+|\-)(\s+\S|\s*$)')
+_enumerated_list_regex = re.compile(
+    r'^(?P<paren>\()?'
+    r'(\d+|#|[ivxlcdm]+|[IVXLCDM]+|[a-zA-Z])'
+    r'(?(paren)\)|\.)(\s+\S|\s*$)')
 
 
 class GoogleDocstring(UnicodeMixin):
-    """Parse Google style docstrings.
-
-    Convert Google style docstrings to reStructuredText.
+    """Convert Google style docstrings to reStructuredText.
 
     Parameters
     ----------
-    docstring : str or list of str
+    docstring : :obj:`str` or :obj:`list` of :obj:`str`
         The docstring to parse, given either as a string or split into
         individual lines.
-    config : sphinx.ext.napoleon.Config or sphinx.config.Config, optional
+    config: :obj:`sphinx.ext.napoleon.Config` or :obj:`sphinx.config.Config`
         The configuration settings to use. If not given, defaults to the
         config object on `app`; or if `app` is not given defaults to the
-        a new `sphinx.ext.napoleon.Config` object.
+        a new :class:`sphinx.ext.napoleon.Config` object.
 
-        See Also
-        --------
-        :class:`sphinx.ext.napoleon.Config`
 
     Other Parameters
     ----------------
-    app : sphinx.application.Sphinx, optional
+    app : :class:`sphinx.application.Sphinx`, optional
         Application object representing the Sphinx process.
-    what : str, optional
+    what : :obj:`str`, optional
         A string specifying the type of the object to which the docstring
         belongs. Valid values: "module", "class", "exception", "function",
         "method", "attribute".
-    name : str, optional
+    name : :obj:`str`, optional
         The fully qualified name of the object.
     obj : module, class, exception, function, method, or attribute
         The object to which the docstring belongs.
-    options : sphinx.ext.autodoc.Options, optional
+    options : :class:`sphinx.ext.autodoc.Options`, optional
         The options given to the directive: an object with attributes
         inherited_members, undoc_members, show_inheritance and noindex that
         are True if the flag option of same name was given to the auto
         directive.
 
+
     Example
     -------
     >>> from sphinx.ext.napoleon import Config
@@ -97,19 +105,20 @@ class GoogleDocstring(UnicodeMixin):
     """
     def __init__(self, docstring, config=None, app=None, what='', name='',
                  obj=None, options=None):
+        # type: (Union[unicode, List[unicode]], SphinxConfig, Sphinx, unicode, unicode, Any, Any) -> None  # NOQA
         self._config = config
         self._app = app
 
         if not self._config:
             from sphinx.ext.napoleon import Config
-            self._config = self._app and self._app.config or Config()
+            self._config = self._app and self._app.config or Config()  # type: ignore
 
         if not what:
             if inspect.isclass(obj):
                 what = 'class'
             elif inspect.ismodule(obj):
                 what = 'module'
-            elif isinstance(obj, collections.Callable):
+            elif isinstance(obj, collections.Callable):  # type: ignore
                 what = 'function'
             else:
                 what = 'object'
@@ -122,11 +131,11 @@ class GoogleDocstring(UnicodeMixin):
             docstring = docstring.splitlines()
         self._lines = docstring
         self._line_iter = modify_iter(docstring, modifier=lambda s: s.rstrip())
-        self._parsed_lines = []
+        self._parsed_lines = []  # type: List[unicode]
         self._is_in_section = False
         self._section_indent = 0
         if not hasattr(self, '_directive_sections'):
-            self._directive_sections = []
+            self._directive_sections = []  # type: List[unicode]
         if not hasattr(self, '_sections'):
             self._sections = {
                 'args': self._parse_parameters_section,
@@ -146,15 +155,17 @@ class GoogleDocstring(UnicodeMixin):
                 'raises': self._parse_raises_section,
                 'references': self._parse_references_section,
                 'see also': self._parse_see_also_section,
+                'todo': self._parse_todo_section,
                 'warning': self._parse_warning_section,
                 'warnings': self._parse_warning_section,
                 'warns': self._parse_warns_section,
                 'yield': self._parse_yields_section,
                 'yields': self._parse_yields_section,
-            }
+            }  # type: Dict[unicode, Callable]
         self._parse()
 
     def __unicode__(self):
+        # type: () -> unicode
         """Return the parsed docstring in reStructuredText format.
 
         Returns
@@ -163,20 +174,22 @@ class GoogleDocstring(UnicodeMixin):
             Unicode version of the docstring.
 
         """
-        return u'\n'.join(self.lines())
+        return u('\n').join(self.lines())
 
     def lines(self):
+        # type: () -> List[unicode]
         """Return the parsed lines of the docstring in reStructuredText format.
 
         Returns
         -------
-        list of str
+        list(str)
             The lines of the docstring in a list.
 
         """
         return self._parsed_lines
 
     def _consume_indented_block(self, indent=1):
+        # type: (int) -> List[unicode]
         lines = []
         line = self._line_iter.peek()
         while(not self._is_section_break() and
@@ -186,6 +199,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_contiguous(self):
+        # type: () -> List[unicode]
         lines = []
         while (self._line_iter.has_next() and
                self._line_iter.peek() and
@@ -194,6 +208,7 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_empty(self):
+        # type: () -> List[unicode]
         lines = []
         line = self._line_iter.peek()
         while self._line_iter.has_next() and not line:
@@ -202,30 +217,29 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _consume_field(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> Tuple[unicode, unicode, List[unicode]]
         line = next(self._line_iter)
 
         before, colon, after = self._partition_field_on_colon(line)
-        _name, _type, _desc = before, '', after
+        _name, _type, _desc = before, '', after  # type: unicode, unicode, unicode
 
         if parse_type:
-            match = _google_typed_arg_regex.match(before)
+            match = _google_typed_arg_regex.match(before)  # type: ignore
             if match:
                 _name = match.group(1)
                 _type = match.group(2)
 
-        if _name[:2] == '**':
-            _name = r'\*\*'+_name[2:]
-        elif _name[:1] == '*':
-            _name = r'\*'+_name[1:]
+        _name = self._escape_args_and_kwargs(_name)
 
         if prefer_type and not _type:
             _type, _name = _name, _type
         indent = self._get_indent(line) + 1
-        _desc = [_desc] + self._dedent(self._consume_indented_block(indent))
-        _desc = self.__class__(_desc, self._config).lines()
-        return _name, _type, _desc
+        _descs = [_desc] + self._dedent(self._consume_indented_block(indent))
+        _descs = self.__class__(_descs, self._config).lines()
+        return _name, _type, _descs
 
     def _consume_fields(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> List[Tuple[unicode, unicode, List[unicode]]]
         self._consume_empty()
         fields = []
         while not self._is_section_break():
@@ -234,11 +248,22 @@ class GoogleDocstring(UnicodeMixin):
                 fields.append((_name, _type, _desc,))
         return fields
 
+    def _consume_inline_attribute(self):
+        # type: () -> Tuple[unicode, List[unicode]]
+        line = next(self._line_iter)
+        _type, colon, _desc = self._partition_field_on_colon(line)
+        if not colon:
+            _type, _desc = _desc, _type
+        _descs = [_desc] + self._dedent(self._consume_to_end())
+        _descs = self.__class__(_descs, self._config).lines()
+        return _type, _descs
+
     def _consume_returns_section(self):
+        # type: () -> List[Tuple[unicode, unicode, List[unicode]]]
         lines = self._dedent(self._consume_to_next_section())
         if lines:
             before, colon, after = self._partition_field_on_colon(lines[0])
-            _name, _type, _desc = '', '', lines
+            _name, _type, _desc = '', '', lines  # type: unicode, unicode, List[unicode]
 
             if colon:
                 if after:
@@ -246,12 +271,7 @@ class GoogleDocstring(UnicodeMixin):
                 else:
                     _desc = lines[1:]
 
-                match = _google_typed_arg_regex.match(before)
-                if match:
-                    _name = match.group(1)
-                    _type = match.group(2)
-                else:
-                    _type = before
+                _type = before
 
             _desc = self.__class__(_desc, self._config).lines()
             return [(_name, _type, _desc,)]
@@ -259,17 +279,27 @@ class GoogleDocstring(UnicodeMixin):
             return []
 
     def _consume_usage_section(self):
+        # type: () -> List[unicode]
         lines = self._dedent(self._consume_to_next_section())
         return lines
 
     def _consume_section_header(self):
+        # type: () -> unicode
         section = next(self._line_iter)
         stripped_section = section.strip(':')
         if stripped_section.lower() in self._sections:
             section = stripped_section
         return section
 
+    def _consume_to_end(self):
+        # type: () -> List[unicode]
+        lines = []
+        while self._line_iter.has_next():
+            lines.append(next(self._line_iter))
+        return lines
+
     def _consume_to_next_section(self):
+        # type: () -> List[unicode]
         self._consume_empty()
         lines = []
         while not self._is_section_break():
@@ -277,23 +307,49 @@ class GoogleDocstring(UnicodeMixin):
         return lines + self._consume_empty()
 
     def _dedent(self, lines, full=False):
+        # type: (List[unicode], bool) -> List[unicode]
         if full:
             return [line.lstrip() for line in lines]
         else:
             min_indent = self._get_min_indent(lines)
             return [line[min_indent:] for line in lines]
 
+    def _escape_args_and_kwargs(self, name):
+        # type: (unicode) -> unicode
+        if name[:2] == '**':
+            return r'\*\*' + name[2:]
+        elif name[:1] == '*':
+            return r'\*' + name[1:]
+        else:
+            return name
+
+    def _fix_field_desc(self, desc):
+        # type: (List[unicode]) -> List[unicode]
+        if self._is_list(desc):
+            desc = [u''] + desc
+        elif desc[0].endswith('::'):
+            desc_block = desc[1:]
+            indent = self._get_indent(desc[0])
+            block_indent = self._get_initial_indent(desc_block)
+            if block_indent > indent:
+                desc = [u''] + desc
+            else:
+                desc = ['', desc[0]] + self._indent(desc_block, 4)
+        return desc
+
     def _format_admonition(self, admonition, lines):
+        # type: (unicode, List[unicode]) -> List[unicode]
         lines = self._strip_empty(lines)
         if len(lines) == 1:
             return ['.. %s:: %s' % (admonition, lines[0].strip()), '']
         elif lines:
             lines = self._indent(self._dedent(lines), 3)
-            return ['.. %s::' % admonition, ''] + lines + ['']
+            return [u'.. %s::' % admonition, u''] + lines + [u'']
         else:
-            return ['.. %s::' % admonition, '']
+            return [u'.. %s::' % admonition, u'']
 
     def _format_block(self, prefix, lines, padding=None):
+        # type: (unicode, List[unicode], unicode) -> List[unicode]
         if lines:
             if padding is None:
                 padding = ' ' * len(prefix)
@@ -309,30 +365,59 @@ class GoogleDocstring(UnicodeMixin):
         else:
             return [prefix]
 
+    def _format_docutils_params(self, fields, field_role='param',
+                                type_role='type'):
+        # type: (List[Tuple[unicode, unicode, List[unicode]]], unicode, unicode) -> List[unicode]  # NOQA
+        lines = []
+        for _name, _type, _desc in fields:
+            _desc = self._strip_empty(_desc)
+            if any(_desc):
+                _desc = self._fix_field_desc(_desc)
+                field = ':%s %s: ' % (field_role, _name)
+                lines.extend(self._format_block(field, _desc))
+            else:
+                lines.append(':%s %s:' % (field_role, _name))
+
+            if _type:
+                lines.append(':%s %s: %s' % (type_role, _name, _type))
+        return lines + ['']
+
     def _format_field(self, _name, _type, _desc):
-        separator = any([s for s in _desc]) and ' --' or ''
+        # type: (unicode, unicode, List[unicode]) -> List[unicode]
+        _desc = self._strip_empty(_desc)
+        has_desc = any(_desc)
+        separator = has_desc and ' -- ' or ''
         if _name:
             if _type:
                 if '`' in _type:
-                    field = ['**%s** (%s)%s' % (_name, _type, separator)]
+                    field = '**%s** (%s)%s' % (_name, _type, separator)  # type: unicode
                 else:
-                    field = ['**%s** (*%s*)%s' % (_name, _type, separator)]
+                    field = '**%s** (*%s*)%s' % (_name, _type, separator)
             else:
-                field = ['**%s**%s' % (_name, separator)]
+                field = '**%s**%s' % (_name, separator)
         elif _type:
             if '`' in _type:
-                field = ['%s%s' % (_type, separator)]
+                field = '%s%s' % (_type, separator)
             else:
-                field = ['*%s*%s' % (_type, separator)]
+                field = '*%s*%s' % (_type, separator)
         else:
-            field = []
-        return field + _desc
+            field = ''
+
+        if has_desc:
+            _desc = self._fix_field_desc(_desc)
+            if _desc[0]:
+                return [field + _desc[0]] + _desc[1:]
+            else:
+                return [field] + _desc
+        else:
+            return [field]
 
     def _format_fields(self, field_type, fields):
+        # type: (unicode, List[Tuple[unicode, unicode, List[unicode]]]) -> List[unicode]
         field_type = ':%s:' % field_type.strip()
         padding = ' ' * len(field_type)
         multi = len(fields) > 1
-        lines = []
+        lines = []  # type: List[unicode]
         for _name, _type, _desc in fields:
             field = self._format_field(_name, _type, _desc)
             if multi:
@@ -342,9 +427,12 @@ class GoogleDocstring(UnicodeMixin):
                     lines.extend(self._format_block(field_type + ' * ', field))
             else:
                 lines.extend(self._format_block(field_type + ' ', field))
+        if lines and lines[-1]:
+            lines.append('')
         return lines
 
     def _get_current_indent(self, peek_ahead=0):
+        # type: (int) -> int
         line = self._line_iter.peek(peek_ahead + 1)[peek_ahead]
         while line != self._line_iter.sentinel:
             if line:
@@ -354,12 +442,21 @@ class GoogleDocstring(UnicodeMixin):
         return 0
 
     def _get_indent(self, line):
+        # type: (unicode) -> int
         for i, s in enumerate(line):
             if not s.isspace():
                 return i
         return len(line)
 
+    def _get_initial_indent(self, lines):
+        # type: (List[unicode]) -> int
+        for line in lines:
+            if line:
+                return self._get_indent(line)
+        return 0
+
     def _get_min_indent(self, lines):
+        # type: (List[unicode]) -> int
         min_indent = None
         for line in lines:
             if line:
@@ -371,9 +468,11 @@ class GoogleDocstring(UnicodeMixin):
         return min_indent or 0
 
     def _indent(self, lines, n=4):
+        # type: (List[unicode], int) -> List[unicode]
         return [(' ' * n) + line for line in lines]
 
     def _is_indented(self, line, indent=1):
+        # type: (unicode, int) -> bool
         for i, s in enumerate(line):
             if i >= indent:
                 return True
@@ -381,7 +480,26 @@ class GoogleDocstring(UnicodeMixin):
                 return False
         return False
 
+    def _is_list(self, lines):
+        # type: (List[unicode]) -> bool
+        if not lines:
+            return False
+        if _bullet_list_regex.match(lines[0]):  # type: ignore
+            return True
+        if _enumerated_list_regex.match(lines[0]):  # type: ignore
+            return True
+        if len(lines) < 2 or lines[0].endswith('::'):
+            return False
+        indent = self._get_indent(lines[0])
+        next_indent = indent
+        for line in lines[1:]:
+            if line:
+                next_indent = self._get_indent(line)
+                break
+        return next_indent > indent
+
     def _is_section_header(self):
+        # type: () -> bool
         section = self._line_iter.peek().lower()
         match = _google_section_regex.match(section)
         if match and section.strip(':') in self._sections:
@@ -396,6 +514,7 @@ class GoogleDocstring(UnicodeMixin):
         return False
 
     def _is_section_break(self):
+        # type: () -> bool
         line = self._line_iter.peek()
         return (not self._line_iter.has_next() or
                 self._is_section_header() or
@@ -404,14 +523,20 @@ class GoogleDocstring(UnicodeMixin):
                     not self._is_indented(line, self._section_indent)))
 
     def _parse(self):
+        # type: () -> None
         self._parsed_lines = self._consume_empty()
+
+        if self._name and (self._what == 'attribute' or self._what == 'data'):
+            self._parsed_lines.extend(self._parse_attribute_docstring())
+            return
+
         while self._line_iter.has_next():
             if self._is_section_header():
                 try:
                     section = self._consume_section_header()
                     self._is_in_section = True
                     self._section_indent = self._get_current_indent()
-                    if _directive_regex.match(section):
+                    if _directive_regex.match(section):  # type: ignore
                         lines = [section] + self._consume_to_next_section()
                     else:
                         lines = self._sections[section.lower()](section)
@@ -425,45 +550,48 @@ class GoogleDocstring(UnicodeMixin):
                     lines = self._consume_to_next_section()
             self._parsed_lines.extend(lines)
 
+    def _parse_attribute_docstring(self):
+        # type: () -> List[unicode]
+        _type, _desc = self._consume_inline_attribute()
+        return self._format_field('', _type, _desc)
+
     def _parse_attributes_section(self, section):
+        # type: (unicode) -> List[unicode]
         lines = []
         for _name, _type, _desc in self._consume_fields():
             if self._config.napoleon_use_ivar:
-                field = ':ivar %s: ' % _name
+                field = ':ivar %s: ' % _name  # type: unicode
                 lines.extend(self._format_block(field, _desc))
                 if _type:
                     lines.append(':vartype %s: %s' % (_name, _type))
             else:
-                lines.append('.. attribute:: ' + _name)
-                if _type:
-                    lines.append('')
-                    if '`' in _type:
-                        lines.append('   %s' % _type)
-                    else:
-                        lines.append('   *%s*' % _type)
-                if _desc:
-                    lines.extend([''] + self._indent(_desc, 3))
+                lines.extend(['.. attribute:: ' + _name, ''])
+                fields = self._format_field('', _type, _desc)
+                lines.extend(self._indent(fields, 3))
                 lines.append('')
         if self._config.napoleon_use_ivar:
             lines.append('')
         return lines
 
     def _parse_examples_section(self, section):
+        # type: (unicode) -> List[unicode]
         use_admonition = self._config.napoleon_use_admonition_for_examples
         return self._parse_generic_section(section, use_admonition)
 
     def _parse_usage_section(self, section):
-        header = ['.. rubric:: Usage:', '']
-        block = ['.. code-block:: python', '']
+        # type: (unicode) -> List[unicode]
+        header = ['.. rubric:: Usage:', '']  # type: List[unicode]
+        block = ['.. code-block:: python', '']  # type: List[unicode]
         lines = self._consume_usage_section()
         lines = self._indent(lines, 3)
         return header + block + lines + ['']
 
     def _parse_generic_section(self, section, use_admonition):
+        # type: (unicode, bool) -> List[unicode]
         lines = self._strip_empty(self._consume_to_next_section())
         lines = self._dedent(lines)
         if use_admonition:
-            header = '.. admonition:: %s' % section
+            header = '.. admonition:: %s' % section  # type: unicode
             lines = self._indent(lines, 3)
         else:
             header = '.. rubric:: %s' % section
@@ -473,63 +601,72 @@ class GoogleDocstring(UnicodeMixin):
             return [header, '']
 
     def _parse_keyword_arguments_section(self, section):
-        return self._format_fields('Keyword Arguments', self._consume_fields())
+        # type: (unicode) -> List[unicode]
+        fields = self._consume_fields()
+        if self._config.napoleon_use_keyword:
+            return self._format_docutils_params(
+                fields,
+                field_role="keyword",
+                type_role="kwtype")
+        else:
+            return self._format_fields('Keyword Arguments', fields)
 
     def _parse_methods_section(self, section):
-        lines = []
+        # type: (unicode) -> List[unicode]
+        lines = []  # type: List[unicode]
         for _name, _, _desc in self._consume_fields(parse_type=False):
             lines.append('.. method:: %s' % _name)
             if _desc:
-                lines.extend([''] + self._indent(_desc, 3))
+                lines.extend([u''] + self._indent(_desc, 3))
             lines.append('')
         return lines
 
     def _parse_note_section(self, section):
+        # type: (unicode) -> List[unicode]
         lines = self._consume_to_next_section()
         return self._format_admonition('note', lines)
 
     def _parse_notes_section(self, section):
+        # type: (unicode) -> List[unicode]
         use_admonition = self._config.napoleon_use_admonition_for_notes
         return self._parse_generic_section('Notes', use_admonition)
 
     def _parse_other_parameters_section(self, section):
+        # type: (unicode) -> List[unicode]
         return self._format_fields('Other Parameters', self._consume_fields())
 
     def _parse_parameters_section(self, section):
+        # type: (unicode) -> List[unicode]
         fields = self._consume_fields()
         if self._config.napoleon_use_param:
-            lines = []
-            for _name, _type, _desc in fields:
-                field = ':param %s: ' % _name
-                lines.extend(self._format_block(field, _desc))
-                if _type:
-                    lines.append(':type %s: %s' % (_name, _type))
-            return lines + ['']
+            return self._format_docutils_params(fields)
         else:
             return self._format_fields('Parameters', fields)
 
     def _parse_raises_section(self, section):
+        # type: (unicode) -> List[unicode]
         fields = self._consume_fields(parse_type=False, prefer_type=True)
         field_type = ':raises:'
         padding = ' ' * len(field_type)
         multi = len(fields) > 1
-        lines = []
+        lines = []  # type: List[unicode]
         for _, _type, _desc in fields:
+            _desc = self._strip_empty(_desc)
             has_desc = any(_desc)
-            sep = (_desc and _desc[0]) and ' -- ' or ''
+            separator = has_desc and ' -- ' or ''
             if _type:
                 has_refs = '`' in _type or ':' in _type
                 has_space = any(c in ' \t\n\v\f ' for c in _type)
 
                 if not has_refs and not has_space:
-                    _type = ':exc:`%s`%s' % (_type, sep)
+                    _type = ':exc:`%s`%s' % (_type, separator)
                 elif has_desc and has_space:
-                    _type = '*%s*%s' % (_type, sep)
+                    _type = '*%s*%s' % (_type, separator)
                 else:
-                    _type = '%s%s' % (_type, sep)
+                    _type = '%s%s' % (_type, separator)
 
                 if has_desc:
-                    field = [_type] + _desc
+                    field = [_type + _desc[0]] + _desc[1:]
                 else:
                     field = [_type]
             else:
@@ -546,10 +683,12 @@ class GoogleDocstring(UnicodeMixin):
         return lines
 
     def _parse_references_section(self, section):
+        # type: (unicode) -> List[unicode]
         use_admonition = self._config.napoleon_use_admonition_for_references
         return self._parse_generic_section('References', use_admonition)
 
     def _parse_returns_section(self, section):
+        # type: (unicode) -> List[unicode]
         fields = self._consume_returns_section()
         multi = len(fields) > 1
         if multi:
@@ -557,7 +696,7 @@ class GoogleDocstring(UnicodeMixin):
         else:
             use_rtype = self._config.napoleon_use_rtype
 
-        lines = []
+        lines = []  # type: List[unicode]
         for _name, _type, _desc in fields:
             if use_rtype:
                 field = self._format_field(_name, '', _desc)
@@ -572,39 +711,51 @@ class GoogleDocstring(UnicodeMixin):
             else:
                 lines.extend(self._format_block(':returns: ', field))
                 if _type and use_rtype:
-                    lines.append(':rtype: %s' % _type)
-                    lines.append('')
+                    lines.extend([':rtype: %s' % _type, ''])
+        if lines and lines[-1]:
+            lines.append('')
         return lines
 
     def _parse_see_also_section(self, section):
+        # type: (unicode) -> List[unicode]
         lines = self._consume_to_next_section()
         return self._format_admonition('seealso', lines)
 
+    def _parse_todo_section(self, section):
+        # type: (unicode) -> List[unicode]
+        lines = self._consume_to_next_section()
+        return self._format_admonition('todo', lines)
+
     def _parse_warning_section(self, section):
+        # type: (unicode) -> List[unicode]
         lines = self._consume_to_next_section()
         return self._format_admonition('warning', lines)
 
     def _parse_warns_section(self, section):
+        # type: (unicode) -> List[unicode]
         return self._format_fields('Warns', self._consume_fields())
 
     def _parse_yields_section(self, section):
+        # type: (unicode) -> List[unicode]
         fields = self._consume_returns_section()
         return self._format_fields('Yields', fields)
 
     def _partition_field_on_colon(self, line):
+        # type: (unicode) -> Tuple[unicode, unicode, unicode]
         before_colon = []
         after_colon = []
         colon = ''
         found_colon = False
-        for i, source in enumerate(_xref_regex.split(line)):
+        for i, source in enumerate(_xref_regex.split(line)):  # type: ignore
             if found_colon:
                 after_colon.append(source)
             else:
-                if (i % 2) == 0 and ":" in source:
+                m = _single_colon_regex.search(source)
+                if (i % 2) == 0 and m:
                     found_colon = True
-                    before, colon, after = source.partition(":")
-                    before_colon.append(before)
-                    after_colon.append(after)
+                    colon = source[m.start(): m.end()]
+                    before_colon.append(source[:m.start()])
+                    after_colon.append(source[m.end():])
                 else:
                     before_colon.append(source)
 
@@ -613,6 +764,7 @@ class GoogleDocstring(UnicodeMixin):
                 "".join(after_colon).strip())
 
     def _strip_empty(self, lines):
+        # type: (List[unicode]) -> List[unicode]
         if lines:
             start = -1
             for i, line in enumerate(lines):
@@ -633,42 +785,38 @@ class GoogleDocstring(UnicodeMixin):
 
 
 class NumpyDocstring(GoogleDocstring):
-    """Parse NumPy style docstrings.
-
-    Convert NumPy style docstrings to reStructuredText.
+    """Convert NumPy style docstrings to reStructuredText.
 
     Parameters
     ----------
-    docstring : str or list of str
+    docstring : :obj:`str` or :obj:`list` of :obj:`str`
         The docstring to parse, given either as a string or split into
         individual lines.
-    config : sphinx.ext.napoleon.Config or sphinx.config.Config, optional
+    config: :obj:`sphinx.ext.napoleon.Config` or :obj:`sphinx.config.Config`
         The configuration settings to use. If not given, defaults to the
         config object on `app`; or if `app` is not given defaults to the
-        a new `sphinx.ext.napoleon.Config` object.
+        a new :class:`sphinx.ext.napoleon.Config` object.
 
-        See Also
-        --------
-        :class:`sphinx.ext.napoleon.Config`
 
     Other Parameters
     ----------------
-    app : sphinx.application.Sphinx, optional
+    app : :class:`sphinx.application.Sphinx`, optional
         Application object representing the Sphinx process.
-    what : str, optional
+    what : :obj:`str`, optional
         A string specifying the type of the object to which the docstring
         belongs. Valid values: "module", "class", "exception", "function",
         "method", "attribute".
-    name : str, optional
+    name : :obj:`str`, optional
         The fully qualified name of the object.
     obj : module, class, exception, function, method, or attribute
         The object to which the docstring belongs.
-    options : sphinx.ext.autodoc.Options, optional
+    options : :class:`sphinx.ext.autodoc.Options`, optional
         The options given to the directive: an object with attributes
         inherited_members, undoc_members, show_inheritance and noindex that
         are True if the flag option of same name was given to the auto
         directive.
 
+
     Example
     -------
     >>> from sphinx.ext.napoleon import Config
@@ -725,34 +873,40 @@ class NumpyDocstring(GoogleDocstring):
 
         Returns
         -------
-        list of str
+        list(str)
             The lines of the docstring in a list.
 
     """
     def __init__(self, docstring, config=None, app=None, what='', name='',
                  obj=None, options=None):
+        # type: (Union[unicode, List[unicode]], SphinxConfig, Sphinx, unicode, unicode, Any, Any) -> None  # NOQA
         self._directive_sections = ['.. index::']
         super(NumpyDocstring, self).__init__(docstring, config, app, what,
                                              name, obj, options)
 
     def _consume_field(self, parse_type=True, prefer_type=False):
+        # type: (bool, bool) -> Tuple[unicode, unicode, List[unicode]]
         line = next(self._line_iter)
         if parse_type:
             _name, _, _type = self._partition_field_on_colon(line)
         else:
             _name, _type = line, ''
         _name, _type = _name.strip(), _type.strip()
+        _name = self._escape_args_and_kwargs(_name)
+
         if prefer_type and not _type:
             _type, _name = _name, _type
-        indent = self._get_indent(line)
-        _desc = self._dedent(self._consume_indented_block(indent + 1))
+        indent = self._get_indent(line) + 1
+        _desc = self._dedent(self._consume_indented_block(indent))
         _desc = self.__class__(_desc, self._config).lines()
         return _name, _type, _desc
 
     def _consume_returns_section(self):
+        # type: () -> List[Tuple[unicode, unicode, List[unicode]]]
         return self._consume_fields(prefer_type=True)
 
     def _consume_section_header(self):
+        # type: () -> unicode
         section = next(self._line_iter)
         if not _directive_regex.match(section):
             # Consume the header underline
@@ -760,6 +914,7 @@ class NumpyDocstring(GoogleDocstring):
         return section
 
     def _is_section_break(self):
+        # type: () -> bool
         line1, line2 = self._line_iter.peek(2)
         return (not self._line_iter.has_next() or
                 self._is_section_header() or
@@ -769,10 +924,11 @@ class NumpyDocstring(GoogleDocstring):
                     not self._is_indented(line1, self._section_indent)))
 
     def _is_section_header(self):
+        # type: () -> bool
         section, underline = self._line_iter.peek(2)
         section = section.lower()
         if section in self._sections and isinstance(underline, string_types):
-            return bool(_numpy_section_regex.match(underline))
+            return bool(_numpy_section_regex.match(underline))  # type: ignore
         elif self._directive_sections:
             if _directive_regex.match(section):
                 for directive_section in self._directive_sections:
@@ -784,6 +940,7 @@ class NumpyDocstring(GoogleDocstring):
                            r" (?P<name2>[a-zA-Z0-9_.-]+))\s*", re.X)
 
     def _parse_see_also_section(self, section):
+        # type: (unicode) -> List[unicode]
         lines = self._consume_to_next_section()
         try:
             return self._parse_numpydoc_see_also_section(lines)
@@ -791,6 +948,7 @@ class NumpyDocstring(GoogleDocstring):
             return self._format_admonition('seealso', lines)
 
     def _parse_numpydoc_see_also_section(self, content):
+        # type: (List[unicode]) -> List[unicode]
         """
         Derived from the NumpyDoc implementation of _parse_see_also.
 
@@ -823,13 +981,13 @@ class NumpyDocstring(GoogleDocstring):
             del rest[:]
 
         current_func = None
-        rest = []
+        rest = []  # type: List[unicode]
 
         for line in content:
             if not line.strip():
                 continue
 
-            m = self._name_rgx.match(line)
+            m = self._name_rgx.match(line)  # type: ignore
             if m and line[m.end():].strip().startswith(':'):
                 push_item(current_func, rest)
                 current_func, line = line[:m.end()], line[m.end():]
@@ -869,12 +1027,12 @@ class NumpyDocstring(GoogleDocstring):
             'const': 'const',
             'attribute': 'attr',
             'attr': 'attr'
-        }
+        }  # type: Dict[unicode, unicode]
         if self._what is None:
-            func_role = 'obj'
+            func_role = 'obj'  # type: unicode
         else:
             func_role = roles.get(self._what, '')
-        lines = []
+        lines = []  # type: List[unicode]
         last_had_desc = True
         for func, desc, role in items:
             if role:
