@@ -9,15 +9,19 @@
     :license: BSD, see LICENSE for details.
 """
 
+import re
+
 from docutils import nodes
 from docutils.transforms import Transform, Transformer
 from docutils.transforms.parts import ContentsFilter
-from docutils.utils import new_document
 from docutils.transforms.universal import SmartQuotes
+from docutils.utils import normalize_language_tag
+from docutils.utils.smartquotes import smartchars
 
 from sphinx import addnodes
 from sphinx.locale import _
 from sphinx.util import logging
+from sphinx.util.docutils import new_document
 from sphinx.util.i18n import format_date
 from sphinx.util.nodes import apply_source_workaround, is_smartquotable
 
@@ -338,12 +342,54 @@ class SphinxContentsFilter(ContentsFilter):
         raise nodes.SkipNode
 
 
-class SphinxSmartQuotes(SmartQuotes):
+class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
     """
     Customized SmartQuotes to avoid transform for some extra node types.
 
     refs: sphinx.parsers.RSTParser
     """
+    def apply(self):
+        # type: () -> None
+        if not self.is_available():
+            return
+
+        SmartQuotes.apply(self)
+
+    def is_available(self):
+        # type: () -> bool
+        builders = self.config.smartquotes_excludes.get('builders', [])
+        languages = self.config.smartquotes_excludes.get('languages', [])
+
+        if self.document.settings.smart_quotes is False:
+            # disabled by 3rd party extension (workaround)
+            return False
+        elif self.config.smartquotes is False:
+            # disabled by confval smartquotes
+            return False
+        elif self.app.builder.name in builders:
+            # disabled by confval smartquotes_excludes['builders']
+            return False
+        elif self.config.language in languages:
+            # disabled by confval smartquotes_excludes['languages']
+            return False
+
+        # confirm selected language supports smart_quotes or not
+        language = self.env.settings['language_code']
+        for tag in normalize_language_tag(language):
+            if tag in smartchars.quotes:
+                return True
+        else:
+            return False
+
+    @property
+    def smartquotes_action(self):
+        # type: () -> unicode
+        """A smartquotes_action setting for SmartQuotes.
+
+        Users can change this setting through :confval:`smartquotes_action`.
+        """
+        return self.config.smartquotes_action
+
     def get_tokens(self, txtnodes):
         # A generator that yields ``(texttype, nodetext)`` tuples for a list
         # of "Text" nodes (interface to ``smartquotes.educate_tokens()``).
@@ -353,3 +399,21 @@ class SphinxSmartQuotes(SmartQuotes):
         for txtnode in txtnodes:
             notsmartquotable = not is_smartquotable(txtnode)
             yield (texttype[notsmartquotable], txtnode.astext())
+
+
+class ManpageLink(SphinxTransform):
+    """Find manpage section numbers and names"""
+    default_priority = 999
+
+    def apply(self):
+        for node in self.document.traverse(addnodes.manpage):
+            manpage = ' '.join([str(x) for x in node.children
+                                if isinstance(x, nodes.Text)])
+            pattern = r'^(?P<path>(?P<page>.+)[\(\.](?P<section>[1-9]\w*)?\)?)$'  # noqa
+            info = {'path': manpage,
+                    'page': manpage,
+                    'section': ''}
+            r = re.match(pattern, manpage)
+            if r:
+                info = r.groupdict()
+            node.attributes.update(info)
