@@ -13,20 +13,19 @@ from __future__ import print_function
 import os
 import re
 from itertools import product
-from subprocess import Popen, PIPE
 from shutil import copyfile
+from subprocess import Popen, PIPE
 
-from six import PY3
 import pytest
+from six import PY3
+from test_build_html import ENV_WARNINGS
 
 from sphinx.errors import SphinxError
-from sphinx.util.osutil import cd, ensuredir
+from sphinx.testing.util import remove_unicode_literals, strip_escseq
 from sphinx.util import docutils
+from sphinx.util.osutil import cd, ensuredir
 from sphinx.writers.latex import LaTeXTranslator
 
-from sphinx.testing.util import SkipTest, remove_unicode_literals, strip_escseq, skip_if
-from test_build_html import ENV_WARNINGS
-
 
 LATEX_ENGINES = ['pdflatex', 'lualatex', 'xelatex']
 DOCCLASSES = ['howto', 'manual']
@@ -37,7 +36,7 @@ STYLEFILES = ['article.cls', 'fancyhdr.sty', 'titlesec.sty', 'amsmath.sty',
 LATEX_WARNINGS = ENV_WARNINGS + """\
 %(root)s/index.rst:\\d+: WARNING: unknown option: &option
 %(root)s/index.rst:\\d+: WARNING: citation not found: missing
-%(root)s/index.rst:\\d+: WARNING: no matching candidate for image URI u'foo.\\*'
+%(root)s/index.rst:\\d+: WARNING: a suitable image for latex builder not found: foo.\\*
 %(root)s/index.rst:\\d+: WARNING: Could not lex literal_block as "c". Highlighting skipped.
 """
 
@@ -77,7 +76,7 @@ def compile_latex_document(app):
                        'SphinxTests.tex'],
                       stdout=PIPE, stderr=PIPE)
         except OSError:  # most likely the latex executable was not found
-            raise SkipTest
+            raise pytest.skip.Exception
         else:
             stdout, stderr = p.communicate()
             if p.returncode != 0:
@@ -90,7 +89,7 @@ def compile_latex_document(app):
 def skip_if_requested(testfunc):
     if 'SKIP_LATEX_BUILD' in os.environ:
         msg = 'Skip LaTeX builds because SKIP_LATEX_BUILD is set'
-        return skip_if(True, msg)(testfunc)
+        return pytest.mark.skipif(True, reason=msg)(testfunc)
     else:
         return testfunc
 
@@ -98,7 +97,7 @@ def skip_if_requested(testfunc):
 def skip_if_stylefiles_notfound(testfunc):
     if kpsetest(*STYLEFILES) is False:
         msg = 'not running latex, the required styles do not seem to be installed'
-        return skip_if(True, msg)(testfunc)
+        return pytest.mark.skipif(True, reason=msg)(testfunc)
     else:
         return testfunc
 
@@ -165,13 +164,15 @@ def test_latex_warnings(app, status, warning):
 
 
 @pytest.mark.sphinx('latex', testroot='basic')
-def test_latex_title(app, status, warning):
+def test_latex_basic(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'test.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\title{The basic Sphinx documentation for testing}' in result
+    assert r'\title{The basic Sphinx documentation for testing}' in result
+    assert r'\release{}' in result
+    assert r'\renewcommand{\releasename}{}' in result
 
 
 @pytest.mark.sphinx('latex', testroot='latex-title')
@@ -184,6 +185,18 @@ def test_latex_title_after_admonitions(app, status, warning):
     assert '\\title{test-latex-title}' in result
 
 
+@pytest.mark.sphinx('latex', testroot='basic',
+                    confoverrides={'release': '1.0'})
+def test_latex_release(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert r'\release{1.0}' in result
+    assert r'\renewcommand{\releasename}{Release}' in result
+
+
 @pytest.mark.sphinx('latex', testroot='numfig',
                     confoverrides={'numfig': True})
 def test_numref(app, status, warning):
@@ -335,6 +348,56 @@ def test_numref_with_language_ja(app, status, warning):
             '\\nameref{\\detokenize{foo:foo}}}') in result
 
 
+@pytest.mark.sphinx('latex', testroot='latex-numfig')
+def test_latex_obey_numfig_is_false(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'numfig_secnum_depth': 0})
+def test_latex_obey_numfig_secnum_depth_is_zero(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,nonumfigreset,mathnumfig]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,nonumfigreset,mathnumfig]{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'numfig_secnum_depth': 2})
+def test_latex_obey_numfig_secnum_depth_is_two(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=2,mathnumfig]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=3,mathnumfig]{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'math_numfig': False})
+def test_latex_obey_numfig_but_math_numfig_false(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=1]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=2]{sphinx}' in result
+
+
 @pytest.mark.sphinx('latex')
 def test_latex_add_latex_package(app, status, warning):
     app.add_latex_package('foo')
@@ -542,8 +605,7 @@ def test_reference_in_caption_and_codeblock_in_footnote(app, status, warning):
     assert ('&\nThis is one more footnote with some code in it %\n'
             '\\begin{footnote}[10]\\sphinxAtStartFootnote\n'
             'Third footnote in longtable\n') in result
-    assert ('\\end{sphinxVerbatim}\n\\let\\sphinxVerbatimTitle\\empty\n'
-            '\\let\\sphinxLiteralBlockLabel\\empty\n%\n\\end{footnote}.\n') in result
+    assert ('\\end{sphinxVerbatim}\n%\n\\end{footnote}.\n') in result
     assert '\\begin{sphinxVerbatim}[commandchars=\\\\\\{\\}]' in result
 
 
@@ -704,7 +766,8 @@ def test_image_in_section(app, status, warning):
     assert ('\\chapter{Another section}' in result)
 
 
-@pytest.mark.sphinx('latex', confoverrides={'latex_logo': 'notfound.jpg'})
+@pytest.mark.sphinx('latex', testroot='basic',
+                    confoverrides={'latex_logo': 'notfound.jpg'})
 def test_latex_logo_if_not_found(app, status, warning):
     try:
         app.builder.build_all()
@@ -724,6 +787,7 @@ def test_toctree_maxdepth_manual(app, status, warning):
     assert '\\setcounter{secnumdepth}' not in result
     assert '\\chapter{Foo}' in result
 
+
 @pytest.mark.sphinx(
     'latex', testroot='toctree-maxdepth',
     confoverrides={'latex_documents': [
@@ -740,6 +804,7 @@ def test_toctree_maxdepth_howto(app, status, warning):
     assert '\\setcounter{secnumdepth}' not in result
     assert '\\section{Foo}' in result
 
+
 @pytest.mark.sphinx(
     'latex', testroot='toctree-maxdepth',
     confoverrides={'master_doc': 'foo'})
@@ -753,6 +818,7 @@ def test_toctree_not_found(app, status, warning):
     assert '\\setcounter{secnumdepth}' not in result
     assert '\\chapter{Foo A}' in result
 
+
 @pytest.mark.sphinx(
     'latex', testroot='toctree-maxdepth',
     confoverrides={'master_doc': 'bar'})
@@ -809,8 +875,8 @@ def test_latex_toplevel_sectioning_is_part(app, status, warning):
     'latex', testroot='toctree-maxdepth',
     confoverrides={'latex_toplevel_sectioning': 'part',
                    'latex_documents': [
-                    ('index', 'Python.tex', 'Sphinx Tests Documentation',
-                     'Georg Brandl', 'howto')
+                       ('index', 'Python.tex', 'Sphinx Tests Documentation',
+                        'Georg Brandl', 'howto')
                    ]})
 def test_latex_toplevel_sectioning_is_part_with_howto(app, status, warning):
     app.builder.build_all()
@@ -839,8 +905,8 @@ def test_latex_toplevel_sectioning_is_chapter(app, status, warning):
     'latex', testroot='toctree-maxdepth',
     confoverrides={'latex_toplevel_sectioning': 'chapter',
                    'latex_documents': [
-                    ('index', 'Python.tex', 'Sphinx Tests Documentation',
-                     'Georg Brandl', 'howto')
+                       ('index', 'Python.tex', 'Sphinx Tests Documentation',
+                        'Georg Brandl', 'howto')
                    ]})
 def test_latex_toplevel_sectioning_is_chapter_with_howto(app, status, warning):
     app.builder.build_all()
@@ -1068,6 +1134,16 @@ def test_latex_index(app, status, warning):
     assert '\n\\index{Einstein}\\index{relativity}\\ignorespaces \nand' in result
 
 
+@pytest.mark.sphinx('latex', testroot='latex-equations')
+def test_latex_equations(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    expected = (app.srcdir / 'expects' / 'latex-equations.tex').text().strip()
+
+    assert expected in result
+
+
 @pytest.mark.sphinx('latex', testroot='image-in-parsed-literal')
 def test_latex_image_in_parsed_literal(app, status, warning):
     app.builder.build_all()
@@ -1076,3 +1152,14 @@ def test_latex_image_in_parsed_literal(app, status, warning):
     assert ('{\\sphinxunactivateextrasandspace \\raisebox{-0.5\\height}'
             '{\\scalebox{2.000000}{\\sphinxincludegraphics[height=1cm]{{pic}.png}}}'
             '}AFTER') in result
+
+
+@pytest.mark.sphinx('latex', testroot='nested-enumerated-list')
+def test_latex_nested_enumerated_list(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    assert r'\setcounter{enumi}{4}' in result
+    assert r'\setcounter{enumii}{3}' in result
+    assert r'\setcounter{enumiii}{9}' in result
+    assert r'\setcounter{enumii}{2}' in result
