@@ -55,7 +55,7 @@ def test_mangle_signature():
         assert res == outp, (u"'%s' -> '%s' != '%s'" % (inp, res, outp))
 
 
-def test_extract_summary():
+def test_extract_summary(capsys):
     from sphinx.util.docutils import new_document
     from mock import Mock
     settings = Mock(language_code='',
@@ -77,6 +77,13 @@ def test_extract_summary():
            'it does not break sentence.']
     assert extract_summary(doc, document) == ' '.join(doc)
 
+    # abbreviations
+    doc = ['Blabla, i.e. bla.']
+    assert extract_summary(doc, document) == 'Blabla, i.e.'
+
+    _, err = capsys.readouterr()
+    assert err == ''
+
 
 @pytest.mark.sphinx('dummy', **default_kw)
 def test_get_items_summary(make_app, app_params):
